package kr.co.vividnext.sodalive.admin.content.curation

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.main.curation.AudioContentCuration
import kr.co.vividnext.sodalive.content.main.curation.QAudioContentCuration.audioContentCuration
import kr.co.vividnext.sodalive.content.main.curation.QAudioContentCurationItem.audioContentCurationItem
import kr.co.vividnext.sodalive.content.main.tab.QAudioContentMainTab.audioContentMainTab
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeries.series
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface AdminContentCurationRepository :
    JpaRepository<AudioContentCuration, Long>,
    AdminContentCurationQueryRepository

interface AdminContentCurationQueryRepository {
    fun getAudioContentCurationList(tabId: Long): List<GetAdminContentCurationResponse>
    fun findByIdAndActive(id: Long): AudioContentCuration?
    fun searchCurationContentItem(curationId: Long, searchWord: String): List<SearchCurationItemResponse>
    fun searchCurationSeriesItem(curationId: Long, searchWord: String): List<SearchCurationItemResponse>
}

@Repository
class AdminContentCurationQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) : AdminContentCurationQueryRepository {
    override fun getAudioContentCurationList(tabId: Long): List<GetAdminContentCurationResponse> {
        return queryFactory
            .select(
                QGetAdminContentCurationResponse(
                    audioContentCuration.id,
                    audioContentMainTab.id,
                    audioContentCuration.title,
                    audioContentCuration.description,
                    audioContentCuration.isAdult,
                    audioContentCuration.isSeries
                )
            )
            .from(audioContentCuration)
            .innerJoin(audioContentCuration.tab, audioContentMainTab)
            .where(
                audioContentCuration.isActive.isTrue,
                audioContentMainTab.id.eq(tabId)
            )
            .orderBy(audioContentCuration.orders.asc())
            .fetch()
    }

    override fun findByIdAndActive(id: Long): AudioContentCuration? {
        return queryFactory
            .selectFrom(audioContentCuration)
            .where(
                audioContentCuration.id.eq(id)
                    .and(audioContentCuration.isActive.isTrue)
            )
            .fetchFirst()
    }

    override fun searchCurationContentItem(
        curationId: Long,
        searchWord: String
    ): List<SearchCurationItemResponse> {
        return queryFactory
            .select(
                QSearchCurationItemResponse(
                    audioContent.id,
                    audioContent.title,
                    audioContent.coverImage.prepend("/").prepend(imageHost)
                )
            )
            .from(audioContent)
            .leftJoin(audioContentCurationItem)
            .on(
                audioContent.id.eq(audioContentCurationItem.content.id)
                    .and(audioContentCurationItem.curation.id.eq(curationId))
            )
            .where(
                audioContent.duration.isNotNull
                    .and(audioContent.member.isNotNull)
                    .and(audioContent.isActive.isTrue)
                    .and(audioContent.title.contains(searchWord))
                    .and(audioContentCurationItem.id.isNull)
            )
            .fetch()
    }

    override fun searchCurationSeriesItem(
        curationId: Long,
        searchWord: String
    ): List<SearchCurationItemResponse> {
        return queryFactory
            .select(
                QSearchCurationItemResponse(
                    series.id,
                    series.title,
                    series.coverImage.prepend("/").prepend(imageHost)
                )
            )
            .from(series)
            .leftJoin(audioContentCurationItem)
            .on(
                series.id.eq(audioContentCurationItem.series.id)
                    .and(audioContentCurationItem.curation.id.eq(curationId))
            )
            .where(
                series.isActive.isTrue
                    .and(series.member.isNotNull)
                    .and(series.title.contains(searchWord))
                    .and(
                        audioContentCurationItem.id.isNull
                            .or(audioContentCurationItem.isActive.isFalse)
                    )
            )
            .fetch()
    }
}
