package kr.co.vividnext.sodalive.agora

import java.io.ByteArrayOutputStream
import java.io.IOException
import java.util.TreeMap

class AccessToken(
    var appId: String,
    private val appCertificate: String,
    val channelName: String,
    private val uid: String,
    var crcChannelName: Int = 0,
    private var crcUid: Int = 0,
    val message: PrivilegeMessage = PrivilegeMessage()
) {

    private lateinit var signature: ByteArray
    private lateinit var messageRawContent: ByteArray

    enum class Privileges(value: Int) {
        JoinChannel(1),
        PublishAudioStream(2),
        PublishVideoStream(3),
        PublishDataStream(4), // For RTM only
        RtmLogin(1000);

        var intValue: Short

        init {
            intValue = value.toShort()
        }
    }

    @Throws(Exception::class)
    fun build(): String {
        if (!AgoraUtils.isUUID(appId)) {
            return ""
        }
        if (!AgoraUtils.isUUID(appCertificate)) {
            return ""
        }

        messageRawContent = AgoraUtils.pack(message)
        signature = generateSignature(
            appCertificate,
            appId,
            channelName,
            uid,
            messageRawContent
        )
        crcChannelName = AgoraUtils.crc32(channelName)
        crcUid = AgoraUtils.crc32(uid)
        val packContent = PackContent(signature, crcChannelName, crcUid, messageRawContent)
        val content: ByteArray = AgoraUtils.pack(packContent)
        return getVersion() + appId + AgoraUtils.base64Encode(content)
    }

    fun addPrivilege(privilege: Privileges, expireTimestamp: Int) {
        message.messages[privilege.intValue] = expireTimestamp
    }

    private fun getVersion(): String {
        return VER
    }

    @Throws(java.lang.Exception::class)
    fun generateSignature(
        appCertificate: String,
        appID: String,
        channelName: String,
        uid: String,
        message: ByteArray
    ): ByteArray {
        val baos = ByteArrayOutputStream()

        try {
            baos.write(appID.toByteArray())
            baos.write(channelName.toByteArray())
            baos.write(uid.toByteArray())
            baos.write(message)
        } catch (e: IOException) {
            e.printStackTrace()
        }

        return AgoraUtils.hmacSign(appCertificate, baos.toByteArray())
    }

    fun fromString(token: String): Boolean {
        if (getVersion() != token.substring(0, AgoraUtils.VERSION_LENGTH)) {
            return false
        }

        try {
            appId = token.substring(AgoraUtils.VERSION_LENGTH, AgoraUtils.VERSION_LENGTH + AgoraUtils.APP_ID_LENGTH)
            val packContent = PackContent()
            AgoraUtils.unpack(
                AgoraUtils.base64Decode(
                    token.substring(
                        AgoraUtils.VERSION_LENGTH + AgoraUtils.APP_ID_LENGTH,
                        token.length
                    )
                ),
                packContent
            )
            signature = packContent.signature
            crcChannelName = packContent.crcChannelName
            crcUid = packContent.crcUid
            messageRawContent = packContent.rawMessage
            AgoraUtils.unpack(messageRawContent, message)
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
            return false
        }

        return true
    }

    class PrivilegeMessage : PackableEx {
        var salt: Int
        var ts: Int
        var messages: TreeMap<Short, Int>

        override fun marshal(out: ByteBuf): ByteBuf {
            return out.put(salt).put(ts).putIntMap(messages)
        }

        override fun unmarshal(input: ByteBuf) {
            salt = input.readInt()
            ts = input.readInt()
            messages = input.readIntMap()
        }

        init {
            salt = AgoraUtils.randomInt()
            ts = AgoraUtils.getTimestamp() + 24 * 3600
            messages = TreeMap()
        }
    }

    class PackContent() : PackableEx {
        var signature: ByteArray = byteArrayOf()
        var crcChannelName = 0
        var crcUid = 0
        var rawMessage: ByteArray = byteArrayOf()

        constructor(signature: ByteArray, crcChannelName: Int, crcUid: Int, rawMessage: ByteArray) : this() {
            this.signature = signature
            this.crcChannelName = crcChannelName
            this.crcUid = crcUid
            this.rawMessage = rawMessage
        }

        override fun marshal(out: ByteBuf): ByteBuf {
            return out
                .put(signature)
                .put(crcChannelName)
                .put(crcUid).put(rawMessage)
        }

        override fun unmarshal(input: ByteBuf) {
            signature = input.readBytes()
            crcChannelName = input.readInt()
            crcUid = input.readInt()
            rawMessage = input.readBytes()
        }
    }

    companion object {
        const val VER = "006"
    }
}
