package kr.co.vividnext.sodalive.audition.applicant

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.audition.AuditionApplicant
import kr.co.vividnext.sodalive.audition.QAuditionApplicant.auditionApplicant
import kr.co.vividnext.sodalive.audition.QAuditionRole.auditionRole
import kr.co.vividnext.sodalive.audition.QAuditionVote.auditionVote
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository

interface AuditionApplicantRepository : JpaRepository<AuditionApplicant, Long>, AuditionApplicantQueryRepository

interface AuditionApplicantQueryRepository {
    fun isAlreadyApplicant(auditionRoleId: Long, memberId: Long): Boolean
    fun getAuditionApplicantTotalCount(auditionRoleId: Long): Int
    fun getAuditionApplicantList(
        auditionRoleId: Long,
        sortType: AuditionApplicantSortType,
        offset: Long,
        limit: Long
    ): List<GetAuditionRoleApplicantItem>

    fun findActiveApplicantByMemberIdAndRoleId(memberId: Long, roleId: Long): AuditionApplicant?
}

class AuditionApplicantQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) : AuditionApplicantQueryRepository {
    override fun isAlreadyApplicant(auditionRoleId: Long, memberId: Long): Boolean {
        return queryFactory
            .select(auditionApplicant.id)
            .from(auditionApplicant)
            .innerJoin(auditionApplicant.role, auditionRole)
            .innerJoin(auditionApplicant.member, member)
            .where(
                auditionRole.id.eq(auditionRoleId),
                member.id.eq(memberId),
                auditionApplicant.isActive.isTrue
            )
            .fetch()
            .size > 0
    }

    override fun getAuditionApplicantTotalCount(auditionRoleId: Long): Int {
        return queryFactory
            .select(auditionApplicant.id)
            .from(auditionApplicant)
            .innerJoin(auditionApplicant.member, member)
            .innerJoin(auditionApplicant.role, auditionRole)
            .where(
                auditionRole.id.eq(auditionRoleId),
                auditionApplicant.isActive.isTrue,
                member.isActive.isTrue
            )
            .fetch()
            .size
    }

    override fun getAuditionApplicantList(
        auditionRoleId: Long,
        sortType: AuditionApplicantSortType,
        offset: Long,
        limit: Long
    ): List<GetAuditionRoleApplicantItem> {
        val orderBy = if (sortType == AuditionApplicantSortType.LIKES) {
            auditionVote.id.count().desc()
        } else {
            auditionApplicant.id.desc()
        }

        return queryFactory
            .select(
                QGetAuditionRoleApplicantItem(
                    auditionApplicant.id,
                    member.id,
                    member.nickname,
                    member.profileImage.prepend("/").prepend(cloudFrontHost),
                    auditionApplicant.voicePath.prepend("/").prepend(cloudFrontHost),
                    auditionVote.id.count()
                )
            )
            .from(auditionApplicant)
            .innerJoin(auditionApplicant.member, member)
            .innerJoin(auditionApplicant.role, auditionRole)
            .leftJoin(auditionVote).on(auditionApplicant.id.eq(auditionVote.applicant.id))
            .where(
                auditionRole.id.eq(auditionRoleId),
                auditionApplicant.isActive.isTrue,
                member.isActive.isTrue
            )
            .groupBy(auditionApplicant.id)
            .orderBy(orderBy)
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    override fun findActiveApplicantByMemberIdAndRoleId(memberId: Long, roleId: Long): AuditionApplicant? {
        return queryFactory
            .selectFrom(auditionApplicant)
            .where(
                auditionApplicant.isActive.isTrue
                    .and(auditionApplicant.member.id.eq(memberId))
                    .and(auditionApplicant.role.id.eq(roleId))
            )
            .fetchFirst()
    }
}
