package kr.co.vividnext.sodalive.content.main.tab.content

import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.content.main.GetAudioContentMainItem
import kr.co.vividnext.sodalive.content.main.GetAudioContentRankingItem
import kr.co.vividnext.sodalive.content.main.banner.AudioContentBannerService
import kr.co.vividnext.sodalive.content.main.curation.AudioContentCurationQueryRepository
import kr.co.vividnext.sodalive.content.main.tab.GetContentCurationResponse
import kr.co.vividnext.sodalive.content.theme.AudioContentThemeQueryRepository
import kr.co.vividnext.sodalive.event.EventService
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.rank.RankingService
import org.springframework.stereotype.Service
import java.time.LocalDateTime

@Service
class AudioContentMainTabContentService(
    private val bannerService: AudioContentBannerService,
    private val audioContentRepository: AudioContentRepository,
    private val audioContentThemeRepository: AudioContentThemeQueryRepository,
    private val rankingService: RankingService,
    private val eventService: EventService,
    private val tagCurationService: ContentMainTabTagCurationService,
    private val curationRepository: AudioContentCurationQueryRepository
) {
    fun fetchData(
        isAdultContentVisible: Boolean,
        contentType: ContentType,
        member: Member
    ): GetContentMainTabContentResponse {
        val memberId = member.id!!
        val isAdult = member.auth != null && isAdultContentVisible
        val tabId = 3L

        // 단편 배너
        val contentBannerList = bannerService.getBannerList(
            tabId = tabId,
            memberId = memberId,
            isAdult = isAdult
        )

        // 새로운 단편 테마
        val themeOfContentList = audioContentThemeRepository.getActiveThemeOfContent(
            isAdult = isAdult,
            contentType = contentType
        ).filter {
            it != "모닝콜" &&
                it != "알람" &&
                it != "슬립콜" &&
                it != "다시듣기" &&
                it != "ASMR" &&
                it != "릴레이" &&
                it != "챌린지" &&
                it != "자기소개"
        }

        // 새로운 단편
        val newContentList = audioContentRepository.findByTheme(
            memberId = member.id!!,
            theme = themeOfContentList,
            isAdult = isAdult,
            contentType = contentType,
            offset = 0,
            limit = 10
        )

        // 일간 랭킹
        val currentDateTime = LocalDateTime.now()
        val dailyRankingStartDate = currentDateTime
            .withHour(15)
            .withMinute(0)
            .withSecond(0)
            .minusDays(2)
        val dailyRankingEndDate = dailyRankingStartDate
            .plusDays(1)

        val rankContentList = rankingService.getContentRanking(
            memberId = memberId,
            isAdult = isAdult,
            contentType = contentType,
            startDate = dailyRankingStartDate,
            endDate = dailyRankingEndDate
        )

        // 이벤트 배너
        val eventBannerList = eventService.getEventList(isAdult = isAdult)

        val contentRankCreatorList = rankingService.fetchCreatorBySellContentCountRankTop20(
            memberId = member.id!!,
            isAdult = isAdult,
            contentType = contentType,
            startDate = dailyRankingStartDate,
            endDate = dailyRankingEndDate
        )

        val likeCountRankContentList = if (contentRankCreatorList.isNotEmpty()) {
            rankingService.fetchCreatorContentByLikeCountTop4(
                creatorId = contentRankCreatorList[0].creatorId,
                isAdult = isAdult,
                contentType = contentType
            )
        } else {
            emptyList()
        }

        val tagList = if (isAdult) {
            tagCurationService.getTagList(isAdult = isAdult, contentType = contentType)
        } else {
            emptyList()
        }

        val tagCurationContentList = if (tagList.isNotEmpty()) {
            tagCurationService.getTagCurationContentList(
                memberId = memberId,
                tag = tagList[0],
                contentType = contentType
            )
        } else {
            emptyList()
        }

        val curationList = curationRepository.findByContentMainTabId(tabId = tabId, isAdult = isAdult)
            .map {
                GetContentCurationResponse(
                    title = it.title,
                    items = audioContentRepository.findAudioContentByCurationIdV2(
                        curationId = it.id!!,
                        memberId = memberId,
                        isAdult = isAdult,
                        contentType = contentType
                    )
                )
            }
            .filter { it.items.isNotEmpty() }

        return GetContentMainTabContentResponse(
            bannerList = contentBannerList,
            contentThemeList = themeOfContentList,
            newContentList = newContentList,
            rankSortTypeList = listOf("매출", "댓글", "좋아요"),
            rankContentList = rankContentList,
            contentRankCreatorList = contentRankCreatorList,
            likeCountRankContentList = likeCountRankContentList,
            salesCountRankContentList = likeCountRankContentList,
            eventBannerList = eventBannerList,
            tagList = tagList,
            tagCurationContentList = tagCurationContentList,
            curationList = curationList
        )
    }

    fun getAudioContentRanking(
        memberId: Long,
        isAdult: Boolean,
        contentType: ContentType,
        sortType: String
    ): List<GetAudioContentRankingItem> {
        val currentDateTime = LocalDateTime.now()
        val dailyRankingStartDate = currentDateTime
            .withHour(15)
            .withMinute(0)
            .withSecond(0)
            .minusDays(2)
        val dailyRankingEndDate = dailyRankingStartDate
            .plusDays(1)

        var loopCount = 0
        var contentList: List<GetAudioContentRankingItem>

        do {
            contentList = rankingService.getContentRanking(
                memberId = memberId,
                isAdult = isAdult,
                contentType = contentType,
                startDate = dailyRankingStartDate.minusDays(loopCount * 5L),
                endDate = dailyRankingEndDate,
                sortType = sortType
            )
            loopCount++
        } while (contentList.size < 5 && loopCount < 5)

        return contentList
    }

    fun getNewContentByTheme(
        theme: String,
        isAdultContentVisible: Boolean,
        contentType: ContentType,
        member: Member
    ): List<GetAudioContentMainItem> {
        val isAdult = member.auth != null && isAdultContentVisible

        val themeList = if (theme.isBlank()) {
            audioContentThemeRepository.getActiveThemeOfContent(isAdult = isAdult, contentType = contentType)
                .filter {
                    it != "모닝콜" &&
                        it != "알람" &&
                        it != "슬립콜" &&
                        it != "다시듣기" &&
                        it != "ASMR" &&
                        it != "릴레이" &&
                        it != "챌린지" &&
                        it != "자기소개"
                }
        } else {
            listOf(theme)
        }

        return audioContentRepository.findByTheme(
            memberId = member.id!!,
            theme = themeList,
            isAdult = isAdult,
            contentType = contentType,
            offset = 0,
            limit = 10
        )
    }

    fun getPopularContentByCreator(
        creatorId: Long,
        isAdult: Boolean,
        contentType: ContentType
    ): List<GetAudioContentRankingItem> {
        return rankingService.fetchCreatorContentByLikeCountTop4(
            creatorId = creatorId,
            isAdult = isAdult,
            contentType = contentType
        )
    }

    fun getRecommendedContentByTag(
        memberId: Long,
        tag: String,
        contentType: ContentType
    ): List<GetAudioContentMainItem> {
        return tagCurationService.getTagCurationContentList(memberId = memberId, tag = tag, contentType = contentType)
    }
}
