package kr.co.vividnext.sodalive.creator.admin.content.series

import com.amazonaws.services.s3.model.ObjectMetadata
import com.fasterxml.jackson.databind.ObjectMapper
import kr.co.vividnext.sodalive.aws.s3.S3Uploader
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.hashtag.HashTag
import kr.co.vividnext.sodalive.content.hashtag.HashTagRepository
import kr.co.vividnext.sodalive.creator.admin.content.series.content.AddingContentToTheSeriesRequest
import kr.co.vividnext.sodalive.creator.admin.content.series.content.RemoveContentToTheSeriesRequest
import kr.co.vividnext.sodalive.creator.admin.content.series.content.SearchContentNotInSeriesResponse
import kr.co.vividnext.sodalive.creator.admin.content.series.genre.CreatorAdminContentSeriesGenreRepository
import kr.co.vividnext.sodalive.creator.admin.content.series.keyword.SeriesKeyword
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.utils.generateFileName
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.web.multipart.MultipartFile

@Service
class CreatorAdminContentSeriesService(
    private val repository: CreatorAdminContentSeriesRepository,
    private val genreRepository: CreatorAdminContentSeriesGenreRepository,
    private val hashTagRepository: HashTagRepository,
    private val audioContentRepository: AudioContentRepository,

    private val s3Uploader: S3Uploader,
    private val objectMapper: ObjectMapper,

    @Value("\${cloud.aws.s3.bucket}")
    private val coverImageBucket: String,

    @Value("\${cloud.aws.cloud-front.host}")
    private val coverImageHost: String
) {
    @Transactional
    fun createSeries(coverImage: MultipartFile?, requestString: String, member: Member) {
        if (coverImage == null) throw SodaException("커버이미지를 선택해 주세요.")

        val request = objectMapper.readValue(requestString, CreateSeriesRequest::class.java)
        val series = request.toSeries()
        val genre = genreRepository.findActiveSeriesGenreById(request.genreId)
        series.genre = genre
        series.member = member
        repository.save(series)

        val keywords = request.keyword
            .replace("#", " #")
            .split(" ")
            .asSequence()
            .map { it.trim() }
            .filter { it.isNotBlank() }
            .map {
                if (!it.startsWith("#")) {
                    "#$it"
                } else {
                    it
                }
            }
            .toSet()
            .map {
                val hashTag = hashTagRepository.findByTag(it)
                    ?: hashTagRepository.save(HashTag(it))

                val seriesKeyword = SeriesKeyword()
                seriesKeyword.series = series
                seriesKeyword.keyword = hashTag

                seriesKeyword
            }
            .toList()

        series.keywordList.addAll(keywords)

        val metadata = ObjectMetadata()
        metadata.contentLength = coverImage.size

        // 커버 이미지 파일명 생성
        val coverImageFileName = generateFileName(prefix = "${series.id}-cover")
        // 커버 이미지 업로드
        val coverImagePath = s3Uploader.upload(
            inputStream = coverImage.inputStream,
            bucket = coverImageBucket,
            filePath = "series_cover/${series.id}/$coverImageFileName",
            metadata = metadata
        )

        series.coverImage = coverImagePath
    }

    @Transactional
    fun modifySeries(coverImage: MultipartFile?, requestString: String, member: Member) {
        val request = objectMapper.readValue(requestString, ModifySeriesRequest::class.java)

        if (
            coverImage == null &&
            request.title == null &&
            request.introduction == null &&
            request.publishedDaysOfWeek == null &&
            request.genreId == null &&
            request.isAdult == null &&
            request.state == null &&
            request.writer == null &&
            request.studio == null &&
            request.isActive == null
        ) {
            throw SodaException("변경사항이 없습니다.")
        }

        val series = repository.findByIdAndCreatorId(id = request.seriesId, creatorId = member.id!!)
            ?: throw SodaException("잘못된 접근입니다.")

        if (coverImage != null) {
            val metadata = ObjectMetadata()
            metadata.contentLength = coverImage.size

            // 커버 이미지 파일명 생성
            val coverImageFileName = generateFileName(prefix = "${series.id}-cover")
            // 커버 이미지 업로드
            val coverImagePath = s3Uploader.upload(
                inputStream = coverImage.inputStream,
                bucket = coverImageBucket,
                filePath = "series_cover/${series.id}/$coverImageFileName",
                metadata = metadata
            )

            series.coverImage = coverImagePath
        }

        if (request.title != null) {
            series.title = request.title
        }

        if (request.introduction != null) {
            series.introduction = request.introduction
        }

        if (request.publishedDaysOfWeek != null) {
            if (
                request.publishedDaysOfWeek.contains(SeriesPublishedDaysOfWeek.RANDOM) &&
                request.publishedDaysOfWeek.size > 1
            ) {
                throw SodaException("랜덤과 연재요일 동시에 선택할 수 없습니다.")
            }

            series.publishedDaysOfWeek.clear()
            series.publishedDaysOfWeek.addAll(request.publishedDaysOfWeek)
        }

        if (request.genreId != null) {
            val genre = genreRepository.findActiveSeriesGenreById(request.genreId)
            series.genre = genre
        }

        if (request.isAdult != null) {
            series.isAdult = request.isAdult
        }

        if (request.state != null) {
            series.state = request.state
        }

        if (request.isActive != null) {
            series.isActive = request.isActive
        }

        if (request.writer != null) {
            series.writer = request.writer
        }

        if (request.studio != null) {
            series.studio = request.studio
        }
    }

    fun getSeriesList(offset: Long, limit: Long, creatorId: Long): GetCreatorAdminContentSeriesListResponse {
        val totalCount = repository.getSeriesCount(creatorId = creatorId)
        val seriesList = repository.getSeriesList(
            offset = offset,
            limit = limit,
            creatorId = creatorId
        )
            .map {
                GetCreatorAdminContentSeriesListItem(
                    seriesId = it.id!!,
                    title = it.title,
                    introduction = it.introduction,
                    coverImageUrl = "$coverImageHost/${it.coverImage!!}",
                    publishedDaysOfWeek = it.publishedDaysOfWeek.toList(),
                    genreId = it.genre!!.id!!,
                    isAdult = it.isAdult,
                    state = it.state,
                    isActive = it.isActive,
                    writer = it.writer,
                    studio = it.studio
                )
            }

        return GetCreatorAdminContentSeriesListResponse(totalCount, seriesList)
    }

    fun getDetail(id: Long, memberId: Long): GetCreatorAdminContentSeriesDetailResponse {
        val series = repository.findByIdAndCreatorId(id = id, creatorId = memberId)
            ?: throw SodaException("잘못된 접근입니다.")

        return series.toDetailResponse(imageHost = coverImageHost)
    }

    fun getSeriesContent(
        seriesId: Long,
        offset: Long,
        limit: Long,
        creatorId: Long
    ): GetCreatorAdminContentSeriesContentResponse {
        val totalCount = repository.getSeriesContentCount(creatorId = creatorId)
        val seriesContentList = repository.getSeriesContentList(
            offset = offset,
            limit = limit,
            seriesId = seriesId,
            creatorId = creatorId,
            imageHost = coverImageHost
        )

        return GetCreatorAdminContentSeriesContentResponse(totalCount, seriesContentList)
    }

    @Transactional
    fun addingContentToTheSeries(request: AddingContentToTheSeriesRequest, memberId: Long) {
        val series = repository.findByIdAndCreatorId(id = request.seriesId, creatorId = memberId)
            ?: throw SodaException("잘못된 접근입니다.")

        val seriesContentList = mutableListOf<SeriesContent>()

        for (contentId in request.contentIdList) {
            val content = audioContentRepository.findByIdAndCreatorId(contentId = contentId, creatorId = memberId)
                ?: continue

            val seriesContent = SeriesContent()
            seriesContent.series = series
            seriesContent.content = content
            seriesContentList.add(seriesContent)
        }

        if (seriesContentList.size > 0) {
            series.contentList.addAll(seriesContentList.toSet())
        } else {
            throw SodaException("추가된 콘텐츠가 없습니다.")
        }
    }

    @Transactional
    fun removeContentInTheSeries(request: RemoveContentToTheSeriesRequest, memberId: Long) {
        val series = repository.findByIdAndCreatorId(id = request.seriesId, creatorId = memberId)
            ?: throw SodaException("잘못된 접근입니다.")

        series.contentList.removeIf { it.content!!.id == request.contentId }
    }

    fun searchContentNotInSeries(
        seriesId: Long,
        searchWord: String,
        memberId: Long
    ): List<SearchContentNotInSeriesResponse> {
        return repository.searchContentNotInSeries(
            seriesId,
            searchWord,
            memberId,
            imageHost = coverImageHost
        )
    }

    @Transactional
    fun updateSeriesOrders(ids: List<Long>) {
        for (index in ids.indices) {
            val series = repository.findByIdOrNull(ids[index])

            if (series != null) {
                series.orders = index + 1
            }
        }
    }
}
